
/**
* @file main.c
*
* @brief electronic dice with LPC1114/FN28
*
* @version 1.0
*
* @date 2013-19-01 
*
* @author  Hartmut Wendt  info@hwhardsoft.de	<http://www.hwhardsoft.de>
*
* @copyright GNU Public License V3
* 
* @note
*   This program is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*   You should have received a copy of the GNU General Public License
*   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifdef __USE_CMSIS
#include "LPC11xx.h"
#endif


// include files
#include "timer16.h"
#include "clkconfig.h"
#include "gpio.h"


// definitions and declarations
#define LED_PORT 0		// Port for led
#define LED_ON 0		// Level to set port to turn on led
#define LED_OFF 1		// Level to set port to turn off led
#define BUTTON_PORT 1   // Port for button

// Program modes
enum {
	PM_Roll_Dice,
	PM_Roll_Out_Dice,
	PM_Show_Dice,
	PM_Power_Down,
};


volatile uint32_t msTicks;                            /* counts 1ms timeTicks */
char Program_Mode;

/********************************************************************//**
 * @brief 		SysTick_Handler
 * @param[in]	None
 * @return		None
 *********************************************************************/
void SysTick_Handler(void) {
  msTicks++;                        /* increment counter necessary in Delay() */
}


/********************************************************************//**
 * @brief 		delays number of tick Systicks (happens every 1 ms)
 * @param[in]	dlyTicks
 * @return		None
 *********************************************************************/
__INLINE static void Delay (uint32_t dlyTicks) {
  uint32_t curTicks;

  curTicks = msTicks;
  while ((msTicks - curTicks) < dlyTicks);
}

// function prototypes for main
void set_dice(unsigned int pips);
void enter_deep_powerdown_mode(void);
char get_button_status(void);
void roll_dice(int *ptrloop, int *ptrdelay_time);

int main(void) {
	// init IO pins
	GPIOInit();

	// Init LED ports for output
	GPIOSetDir( LED_PORT, 3, 1 );	// set Port 0.3
	GPIOSetDir( LED_PORT, 4, 1 );	// set Port 0.4
	GPIOSetDir( LED_PORT, 7, 1 );	// set Port 0.7
	GPIOSetDir( LED_PORT, 8, 1 );	// set Port 0.8
	GPIOSetDir( LED_PORT, 9, 1 );	// set Port 0.9
	GPIOSetDir( LED_PORT, 5, 1 );	// set Port 0.5
	GPIOSetDir( LED_PORT, 6, 1 );	// set Port 0.6

	// Init Button port for input
	GPIOSetDir( BUTTON_PORT, 4, 0);	// set Port 1.4	as input

	set_dice(0);

	// configure sysTick Timer
	  if (SysTick_Config(SystemCoreClock / 1000)) { /* Setup SysTick Timer for 1 msec interrupts  */
	    while (1);                                  /* Capture error */
	  }


	// Enter an infinite loop, just incrementing a counter
	int iloops = 0 ;
    int delay_time = 10; // start with 50ms

	// check button status for program_mode init
	if (get_button_status()) Program_Mode = PM_Roll_Dice; // button pressed -> roll dice
	else Program_Mode = PM_Power_Down; // button unpressed -> go to deep sleep
    // Program_Mode = PM_Roll_Dice;

	// infinite loop with state machine
	while(1) {
		switch (Program_Mode) {

			case PM_Roll_Dice:
				// show the different dice pips continous
				roll_dice(&iloops, &delay_time);
				// roll out after button release
				if  (!get_button_status()) Program_Mode = PM_Roll_Out_Dice;
				break;

			case PM_Roll_Out_Dice:
				// increase the speed
				delay_time *= 2;
				// show the different dice pips continous
				roll_dice(&iloops, &delay_time);
				// roll dice again if button is pressed
				if  (get_button_status()) {
					iloops = 0;
					delay_time = 10;
					Program_Mode = PM_Roll_Dice;
					break;
				}
				// stop roll out if delay_time exceed 1s
				if (delay_time > 1000) {
					iloops = 0;
					Program_Mode = PM_Show_Dice;
				}
				break;

			case PM_Show_Dice:
				while(iloops < 50) {
					Delay(100); 	// wait 100ms
					iloops++;
					// roll dice again if button is pressed
					if  (get_button_status()) {
						iloops = 0;
						delay_time = 10;
						Program_Mode = PM_Roll_Dice;
						break;
					}
				}
				if (iloops >= 50) Program_Mode = PM_Power_Down;
				break;

			case PM_Power_Down:
				// switch off LED's
				set_dice(0);
				// set variables and program mode after wakeup
				iloops = 0;
				delay_time = 10;
				Program_Mode = PM_Roll_Dice;
				enter_deep_powerdown_mode();
				break;
		}

	}
	return 0 ;
}

/********************************************************************//**
 * @brief 		Set Pips for dice
 * @param[in]	pips  specifies the actual pips of the dice
 *                    -0 : set all LEDs off
 *                    -1...6 : set LEDs for corresponding pips
 * @return		None
 *********************************************************************/
void set_dice(unsigned int pips) {
	switch (pips) {

	case 1:
		GPIOSetValue(LED_PORT,3,LED_OFF);
		GPIOSetValue(LED_PORT,4,LED_OFF);
		GPIOSetValue(LED_PORT,7,LED_OFF);
		GPIOSetValue(LED_PORT,8,LED_ON);
		GPIOSetValue(LED_PORT,9,LED_OFF);
		GPIOSetValue(LED_PORT,5,LED_OFF);
		GPIOSetValue(LED_PORT,6,LED_OFF);
		break;

	case 2:
		GPIOSetValue(LED_PORT,3,LED_OFF);
		GPIOSetValue(LED_PORT,4,LED_OFF);
		GPIOSetValue(LED_PORT,7,LED_ON);
		GPIOSetValue(LED_PORT,8,LED_OFF);
		GPIOSetValue(LED_PORT,9,LED_OFF);
		GPIOSetValue(LED_PORT,5,LED_OFF);
		GPIOSetValue(LED_PORT,6,LED_ON);
		break;

	case 3:
		GPIOSetValue(LED_PORT,3,LED_OFF);
		GPIOSetValue(LED_PORT,4,LED_OFF);
		GPIOSetValue(LED_PORT,7,LED_ON);
		GPIOSetValue(LED_PORT,8,LED_ON);
		GPIOSetValue(LED_PORT,9,LED_OFF);
		GPIOSetValue(LED_PORT,5,LED_OFF);
		GPIOSetValue(LED_PORT,6,LED_ON);
		break;

	case 4:
		GPIOSetValue(LED_PORT,3,LED_ON);
		GPIOSetValue(LED_PORT,4,LED_OFF);
		GPIOSetValue(LED_PORT,7,LED_ON);
		GPIOSetValue(LED_PORT,8,LED_OFF);
		GPIOSetValue(LED_PORT,9,LED_ON);
		GPIOSetValue(LED_PORT,5,LED_OFF);
		GPIOSetValue(LED_PORT,6,LED_ON);
		break;

	case 5:
		GPIOSetValue(LED_PORT,3,LED_ON);
		GPIOSetValue(LED_PORT,4,LED_OFF);
		GPIOSetValue(LED_PORT,7,LED_ON);
		GPIOSetValue(LED_PORT,8,LED_ON);
		GPIOSetValue(LED_PORT,9,LED_ON);
		GPIOSetValue(LED_PORT,5,LED_OFF);
		GPIOSetValue(LED_PORT,6,LED_ON);
		break;

	case 6:
		GPIOSetValue(LED_PORT,3,LED_ON);
		GPIOSetValue(LED_PORT,4,LED_ON);
		GPIOSetValue(LED_PORT,7,LED_ON);
		GPIOSetValue(LED_PORT,8,LED_OFF);
		GPIOSetValue(LED_PORT,9,LED_ON);
		GPIOSetValue(LED_PORT,5,LED_ON);
		GPIOSetValue(LED_PORT,6,LED_ON);
		break;

	default:
		// all off
		GPIOSetValue(LED_PORT,3,LED_OFF);
		GPIOSetValue(LED_PORT,4,LED_OFF);
		GPIOSetValue(LED_PORT,7,LED_OFF);
		GPIOSetValue(LED_PORT,8,LED_OFF);
		GPIOSetValue(LED_PORT,9,LED_OFF);
		GPIOSetValue(LED_PORT,5,LED_OFF);
		GPIOSetValue(LED_PORT,6,LED_OFF);
	}
}



/********************************************************************//**
 * @brief 		roll the dice continuously
 * @param[in]	None
 * @return		None
 *********************************************************************/
void roll_dice(int *ptrloop, int *ptrdelay_time) {
	if (*ptrloop >= 6) *ptrloop=0;
	(*ptrloop)++;
	set_dice(*ptrloop);
	Delay(*ptrdelay_time);
}



/********************************************************************//**
 * @brief 		Enter deep power-down mode
 * @param[in]	None
 * @return		None
 *********************************************************************/
void enter_deep_powerdown_mode(void) {
    //Specify DPDEN to power control register
    LPC_PMU->PCON = (1<<1)|(1<<11);

    //Specify deep power-down before entering mode
    SCB->SCR |= (1<<2);  // set SLEEPDEEP bit

    // Enable IRC before deep power-down mode
    LPC_SYSCON->PDRUNCFG &= ~((1<<0) | (1<<0));

    // Enter deep power down mode
    __WFI();
}

/********************************************************************//**
 * @brief 		get the status (pressed / unpressed) of a button at P1.4
 * @param[in]	None
 * @return		status (0 = unpressed, 1 = pressed)
 *********************************************************************/
char get_button_status(void) {
	if (LPC_GPIO1->DATA & (1<<4)) return(0);
	else return(1);
}

